myname = 'compare_seas_avg_1-16_v4.R'

mpostscript( myname, horiz=FALSE )

ppp_x = 3
ppp_y = 4

plots_per_page = ppp_x * ppp_y

source( '~/location.R' )
source('../mod_descrip_v10.R')
source('../make_all_filenames_v4.R')

source('~/src/mine/R/multi_page_png.R' )

seas = c( 'DJF', 'MAM', 'JJA', 'SON' )

#======================================================================================
do_mod_var = function( fn_all, modname, imod, varname_cmip5, varname_livneh, ivar, units_tag ) {

	fname_mod_ds = paste0( '../', 
		fn_all$dstype[[IDSTYPE_16DEG]]$mod[[imod]]$var[[ivar]]$expt[[IEXPT_HISTORICAL]]$period[[IPERIOD_TRAINING_PERIOD]]$ds
		)

	nseas = length( seas )
	fname_mod = array( ' ', nseas )
	for( iseas in 1:nseas ) {
		seas_tag = paste0( 'srs.ds.postdsbc.', seas[iseas], '.all.nc' )
		fname_mod[iseas] = sub( 'srs.ds.nc', seas_tag, fname_mod_ds )
		if( filesize( fname_mod[iseas] ) < 1000 ) 
			stop(paste("File not found:", fname_mod[iseas] ))
		print( fname_mod[iseas] )
		}

	# Prec.1950-1999.kg_m2_sec.crop.ptiles.nozero.nc

	if( varname_cmip5 == 'pr' ) {
		scalefact = 86400.	# convert kg/m**2/s to mm/day
		offset = 0.0
		dv = make_discretevals( 'prism_precip', 
			c(0.0, 0.1,0.2,0.5,1.0,2,3,4,5,6,7,8,10,12,14,16) )
		do_ratio = TRUE
		}

	else if( varname_cmip5 == 'tasmin' ) {
		scalefact = 1.0
		offset = -273.15
		dv = make_discretevals( 'prism_temp',
			seq(-25,30,by=2.5))
		dv$values2label = seq(-40,30,by=10)
		do_ratio = FALSE
		}

	else if( varname_cmip5 == 'tasmax' ) {
		scalefact = 1.0
		offset = -273.15
		dv = make_discretevals( 'prism_temp',
			seq(-15,40,by=2.5))
		dv$values2label = seq(-30,50,by=10)
		do_ratio = FALSE
		}

	else if( varname_cmip5 == 'DTR' ) {
		scalefact = 1.0
		offset = 0.0
		dv = make_discretevals( 'idl',
			seq(0,24,by=2))
		do_ratio = FALSE
		}

	else
		stop(paste("uncaught case for var", varname_cmip5))

	fname_obs = array( ' ', nseas )
	for( iseas in 1:nseas ) {
		fname_obs[iseas] = paste0( '../../Livneh/livneh_NAmerExt_15Oct2014/',
			varname_livneh, '.1950-2005.', units_tag, '.crop.preproc_obs.', 
			varname_livneh, '_only.monthly.', seas[iseas], '.all.nc' )
		if( filesize( fname_obs[iseas] ) < 1000 ) 
			stop(paste("File not found:", fname_obs[iseas] ))
		print(fname_obs[iseas] )
		}

	for( iseas in 1:nseas ) {

		ncid_mod = nc_open( fname_mod[iseas] )
		ncid_obs = nc_open( fname_obs[iseas] )

		mppng_source_file( fname_mod[iseas] )

		dat_mod = (ncvar_get( ncid_mod ) + offset) * scalefact
		dat_obs = (ncvar_get( ncid_obs ) + offset) * scalefact

		var_mod = ncid_mod$var[[1]]
		xvals_mod = var_mod$dim[[1]]$vals
		if( max( xvals_mod ) > 181 ) xvals_mod = xvals_mod - 360
		yvals_mod = var_mod$dim[[2]]$vals

		var_obs = ncid_obs$var[[1]]
		xvals_obs = var_obs$dim[[1]]$vals
		if( max( xvals_obs ) > 181 ) xvals_obs = xvals_obs - 360
		yvals_obs = var_obs$dim[[2]]$vals

		xlim = range( xvals_mod )
		ylim = range( yvals_mod )
		ylim[2] = 56

		#------------------------------------
		# Data shape is [nx,ny]
		#------------------------------------
		zrange = range( dat_obs )

		main = paste0(varname_cmip5, ' ', seas[iseas] )
		print(main)

		cfh = 1.2
		cfy = 0.6

		ccontour( xvals_obs, yvals_obs, dat_obs,
			raster=TRUE,
			xlim=xlim, ylim=ylim, 
			zrange=zrange,
			discretevals=dv,
			contour_show=FALSE,
			cbar_fudge_height=cfh,
			cbar_fudge_yoffset=cfy,
			main=paste('Obs mean',main) )
		col_map = grey(0.5)
		map('state', add=TRUE, col=col_map )
		mppng_next_frame()

		ccontour( xvals_mod, yvals_mod, dat_mod,
			raster=TRUE,
			xlim=xlim, ylim=ylim, 
			discretevals=dv,
			zrange=zrange,
			contour_show=FALSE,
			cbar_fudge_yoffset=cfy,
			cbar_fudge_height=cfh,
			main=paste(modname,'mean',main) )
		map('state', add=TRUE, col=col_map )
		mppng_next_frame()

		if( do_ratio ) {
			rv = calc_2d_ratio( xvals_mod, yvals_mod, dat_mod,
				xvals_obs, yvals_obs, dat_obs,
				diffratio_pct=TRUE )
			main = 'Model error w.r.t. obs [%]'
			dv_err = make_discretevals( 'pretty', seq(-45,45,by=10))
			}
		else
			{
			rv = calc_2d_ratio( xvals_mod, yvals_mod, dat_mod,
				xvals_obs, yvals_obs, dat_obs,
				just_difference=TRUE )
			main = 'Model error w.r.t. obs [degC]'
			dv_err = make_discretevals( 'pretty', seq(-0.9,0.9,by=0.2))
			}

		ccontour( rv$xvals, rv$yvals, rv$ratio,
			xlim=xlim, ylim=ylim, 
			raster=TRUE,
			discretevals=dv_err,
			contour_show=FALSE,
			cbar_fudge_height=cfh,
			cbar_fudge_yoffset=cfy,
			main=main )
		map('state', add=TRUE, col=col_map )

		terr = rv$ratio
		terr[ is.nan     ( terr ) ] = NA
		terr[ is.infinite( terr ) ] = NA
		merr = mean( terr, na.rm=TRUE )
		rmse = sqrt( mean( terr*terr, na.rm=TRUE ))

		str = paste0( 'RMSE: ', mformatC('%.1f',rmse), '  Mean: ', mformatC('%.1f',merr))
		print(str)
		text( min(xlim), max(ylim), str, adj=c(0,1.3), cex=1.2 )
		mppng_next_frame()

		nc_close( ncid_mod )
		nc_close( ncid_obs )
		}
}

#======================================================================================
doitfor = function( fn_all, modd, modname, imod ) {

	nvar = length( modd$var )

	for( ivar in 1:nvar ) {
		varname_cmip5  = modd$var[[ivar]]$varname_cmip5  
		varname_livneh = modd$var[[ivar]]$varname_livneh 
		units_tag      = modd$var[[ivar]]$units_tag
		do_mod_var( fn_all, modname, imod, varname_cmip5, varname_livneh, ivar, units_tag )
		}
}

#======================================================================================
calc_dat = function( opts ) {

	dat = list()

        modd = make_mod_descrip( location, LOC_NASA, LOC_CIRRUS )
	nmod = length( modd$model )

	#-----------------------------
	# Get flat list of model names
	#-----------------------------
	for( imod in 1:nmod ) {
		if( imod == 1 )
			modnames = modd$model[[imod]]$modname_cmip5
		else
			modnames = c( modnames, modd$model[[imod]]$modname_cmip5 )
		}

	#---------------------------
	# Get flat list of var names
	#---------------------------
	nvar = length( modd$var )
	for( ivar in 1:nvar ) {
		if( ivar == 1 ) 
			varnames = modd$var[[ivar]]$varname_cmip5
		else
			varnames = c( varnames, modd$var[[ivar]]$varname_cmip5 )
		}

	hierarchy = list( modnames, varnames )

	npages_tot = nmod*nvar

	#---------------------------------------------------
	# Set up multiple png plots per page in a html bunch
	#---------------------------------------------------
	multi_page_png( myname, plots_per_page, npages_tot,
		width=900, height=1000,
		header_html='Seasonal averages of Livneh Obs vs. LOCA downscaled (1/16th deg)',
		hierarchy=hierarchy )

	#-----------------------------------------------
	# Must do 'par' AFTER the call to multi_page_png
	#-----------------------------------------------
	par(mfrow=c(ppp_y,ppp_x), mar=c(3,1,2.2,0.5), mgp=c(3,0,0))

	fn_all = make_all_filenames( modd )

	for( imod in 1:nmod ) {
		doitfor( fn_all, modd, modd$model[[imod]]$modname_cmip5, imod )
		}

	return(dat)
}

#======================================================================================
plotup = function( dat, opts, po ) {

}

#======================================================================================
doit = function() {

	use_cache = FALSE
	cache_fname = paste(myname,'_Rcache', sep='' )
	cache_file_exists = (filesize(cache_fname) > 10)

	#-----------------
	# Set data options
	#-----------------
	opts = list()

	#--------------------
	# Use cache, if asked
	#--------------------
	if( use_cache ) {
		if( cache_file_exists ) {
			print(paste("loading data from cache file", cache_fname))
			load( file=cache_fname )
			print('...done')
			do_calc_dat = FALSE
			}
		else
			do_calc_dat = TRUE
		}
	else
		do_calc_dat = TRUE

	#-------------------
	# Calculate the data
	#-------------------
	if( do_calc_dat )
		dat = calc_dat( opts )
		
	#-----------------------------------------------------
	# Save to cache if requested and doesn't exist already
	#-----------------------------------------------------
	if( use_cache && (! cache_file_exists))
		save( file=cache_fname, dat, opts )
			
	#---------------------------------
	# Set plot options and plot result
	#---------------------------------
	po = list()
	plotup( dat, opts, po )
}

#======================================================================================

doit()
mdev.off( myname )
